<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
	xmlns:data="http://example.com/data/dataset" xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:sawsdl="http://www.w3.org/ns/sawsdl"
	xmlns:fn="http://www.w3.org/2005/xpath-functions" exclude-result-prefixes="xs" version="2.0">
	<xsl:output method="xml" encoding="UTF-8" media-type="application/rdf+xml" indent="yes"/>
	<!-- XPath 2.0 functions replaced by simpler expressions or XPath 1.0 functions for portability. Opera 10 does not seem to support concat(). Unbelievable. -->
	<!-- Find the URL of the associated schema from the XML document, and load the schema document into a variable -->
	<!--xsl:variable name="schemaUrl" select="substring-after(string(/*/@xsi:schemaLocation)[1], ' ')" /-->
	<xsl:variable name="schemaLocationString"><xsl:value-of select="/*/@xsi:schemaLocation"></xsl:value-of></xsl:variable>
	<xsl:variable name="schemaUrl" select="substring-after($schemaLocationString, ' ')"/>
	<xsl:variable name="schemaDoc" select="document($schemaUrl)"/>
	<!-- doc() -->
	<!-- Find the target namespace from the schema, and store a character (or nothing) as a path separator -->
	<!--xsl:variable name="targetNamespace" select="$schemaDoc/xs:schema/@targetNamespace"/-->
	<xsl:variable name="targetNamespace"><xsl:value-of select="$schemaDoc/xs:schema/@targetNamespace"/></xsl:variable>
	<xsl:variable name="targetNamespaceEndCharacter" as="xs:string">
		<xsl:choose>
			<!--xsl:when test="ends-with($targetNamespace, '/')"-->
			<xsl:when test="substring($targetNamespace, string-length($targetNamespace), 1) = '/'">
				<xsl:text/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text>/</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	<xsl:variable name="listElements" select="$schemaDoc//xs:element[xs:simpleType/xs:list]" />
	<!-- Create an RDF document, filling it with data from the XML dataset. -->
	<xsl:template match="/">
		<rdf:RDF xmlns="http://example.com/schema#">
			<!--xsl:comment select="concat('$schemaUrl is ', $schemaUrl)"/-->
			<xsl:comment><xsl:value-of select="'$schemaUrl is '" /><xsl:value-of select="$schemaUrl"/></xsl:comment>
			<!--xsl:comment select="concat('$targetNamespace is ', $targetNamespace)"/-->
			<xsl:comment><xsl:value-of select="'$targetNamespace is '" /><xsl:value-of select="$targetNamespace"/></xsl:comment>
			<xsl:apply-templates/>
		</rdf:RDF>
	</xsl:template>
	<xsl:template match="data:dataSet">
		<xsl:apply-templates/>
	</xsl:template>
	<xsl:template match="data:record">
		<xsl:element name="{local-name(.)}" namespace="{$targetNamespace}">
			<!--xsl:attribute name="rdf:about" select="concat($targetNamespace,$targetNamespaceEndCharacter,dc:identifier)"/-->
			<xsl:attribute name="rdf:about">
				<xsl:value-of select="$targetNamespace" /><xsl:value-of select="$targetNamespaceEndCharacter" /><xsl:value-of select="dc:identifier" />
			</xsl:attribute>
			<xsl:apply-templates/>
		</xsl:element>
	</xsl:template>
	<xsl:template match="dc:identifier"/>
	<xsl:template match="dc:title">
		<xsl:element name="rdfs:label">
			<xsl:value-of select="."/>
		</xsl:element>
	</xsl:template>
	<xsl:template match="*">
		<xsl:variable name="elementName" select="local-name(.)"/>
		<xsl:variable name="elementDefinition" select="$schemaDoc//xs:element[@name = $elementName]"/>
		<!--xsl:variable name="elementType" select="$elementDefinition/substring-after(@type, ':')"/-->
		<xsl:variable name="elementType" select="$elementDefinition/@type"/>
		<xsl:variable name="elementTypeUnqualified" select="substring-after($elementType, ':')"/>
		<xsl:element name="{$elementName}" namespace="{$targetNamespace}">
			<xsl:attribute name="rdf:datatype">
				<!--xsl:value-of select="concat('http://www.w3.org/2001/XMLSchema#',$elementTypeUnqualified)"/-->
				<xsl:value-of select="'http://www.w3.org/2001/XMLSchema#'"/><xsl:value-of select="$elementTypeUnqualified"/>
			</xsl:attribute>
			<!--xsl:if test="exists($elementDefinition/@sawsdl:modelReference)"-->
			<xsl:if test="$elementDefinition/@sawsdl:modelReference">
				<xsl:attribute name="rdf:resource" select="$elementDefinition/@sawsdl:modelReference"/>
			</xsl:if>
			<xsl:apply-templates/>
		</xsl:element>
	</xsl:template>
	<!-- Match all elements whose definition in the schema contains a list datatype. -->
	<xsl:template match="*[local-name(.) = $listElements/descendant-or-self::*/@name]">
		<xsl:variable name="elementName" select="local-name(.)"/>
		<xsl:variable name="elementDefinition" select="$schemaDoc//xs:element[@name = $elementName]"/>
		<xsl:variable name="elementType" select="$elementDefinition/xs:simpleType/xs:list/@itemType"/>
		<xsl:variable name="elementTypeUnqualified" select="substring-after($elementType, ':')"/>
		<xsl:element name="{$elementName}" namespace="{$targetNamespace}">
			<xsl:attribute name="rdf:parseType" select="'Collection'" />
			<!-- For each item in the list, create a new child element containing its content. -->
			<xsl:for-each select="fn:distinct-values(.)">
				<xsl:element name="rdf:Description">
					<xsl:attribute name="rdf:about" select="concat($targetNamespace,$targetNamespaceEndCharacter,.)" />
				</xsl:element>
			</xsl:for-each>
		</xsl:element>
	</xsl:template>
</xsl:stylesheet>
